@@ -21,7 +21,11 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <fcntl.h>
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
 #include <limits.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -332,7 +336,9 @@ cm_locate_xmlrpc_service(const char *server,
 /* Make an XML-RPC request to the "cert_request" method. */
 static int
 submit_or_poll_uri(const char *uri, const char *cainfo, const char *capath,
-	           const char *csr, const char *reqprinc, const char *profile)
+		   const char *uid, const char *pwd, const char *csr,
+		   const char *reqprinc, const char *profile,
+		   const char *issuer)
 {
 	struct cm_submit_x_context *ctx;
 	const char *args[2];
@@ -345,10 +351,18 @@ submit_or_poll_uri(const char *uri, const char *cainfo, const char *capath,
 
 	/* Prepare to make an XML-RPC request. */
 submit:
-	ctx = cm_submit_x_init(NULL, uri, "cert_request",
-			       cainfo, capath,
-			       cm_submit_x_negotiate_on,
-			       cm_submit_x_delegate_on);
+	if ((uid != NULL) && (pwd != NULL) &&
+	    (strlen(uid) > 0) && (strlen(pwd) > 0)) {
+		ctx = cm_submit_x_init(NULL, uri, "cert_request",
+				       cainfo, capath, uid, pwd,
+				       cm_submit_x_negotiate_off,
+				       cm_submit_x_delegate_off);;
+	} else {
+		ctx = cm_submit_x_init(NULL, uri, "cert_request",
+				       cainfo, capath, NULL, NULL,
+				       cm_submit_x_negotiate_on,
+				       cm_submit_x_delegate_on);
+	}
 	if (ctx == NULL) {
 		fprintf(stderr, "Error setting up for XMLRPC to %s on "
 			"the client.\n", uri);
@@ -366,6 +380,10 @@ submit:
 	if (profile != NULL) {
 		cm_submit_x_add_named_arg_s(ctx, "profile_id", profile);
 	}
+	/* Add the requested CA issuer named argument. */
+	if (issuer != NULL) {
+		cm_submit_x_add_named_arg_s(ctx, "ca", issuer);
+	}
 	/* Tell the server to add entries for a principal if one
 	 * doesn't exist yet. */
 	cm_submit_x_add_named_arg_b(ctx, "add", 1);
@@ -382,6 +400,14 @@ submit:
 		switch (i / 1000) {
 		case 2: /* authorization error - permanent */
 		case 3: /* invocation error - permanent */
+			if ((i == 3005) && (issuer != NULL)) {
+				/* Most likely the server didn't understand the
+				 * "ca" argument.  At least, at this
+				 * point.  Randomly dropping arguments is not
+				 * really an extensible solution, though. */
+				issuer = NULL;
+				goto submit;
+			}
 			if ((i == 3005) && (profile != NULL)) {
 				/* Most likely the server didn't understand the
 				 * "profile_id" argument.  At least, at this
@@ -440,12 +466,14 @@ static int
 submit_or_poll(const char *uri, const char *cainfo, const char *capath,
 	       const char *server, int ldap_uri_cmd, const char *ldap_uri,
 	       const char *host, const char *domain, char *basedn,
-	       const char *csr, const char *reqprinc, const char *profile)
+	       const char *uid, const char *pwd, const char *csr,
+	       const char *reqprinc, const char *profile, const char *issuer)
 {
 	int i, u;
 	char **uris;
 
-	i = submit_or_poll_uri(uri, cainfo, capath, csr, reqprinc, profile);
+	i = submit_or_poll_uri(uri, cainfo, capath, uid, pwd, csr, reqprinc,
+			       profile, issuer);
 	if ((i == CM_SUBMIT_STATUS_UNREACHABLE) ||
 	    (i == CM_SUBMIT_STATUS_UNCONFIGURED)) {
 		u = cm_locate_xmlrpc_service(server, ldap_uri_cmd, ldap_uri,
@@ -456,7 +484,8 @@ submit_or_poll(const char *uri, const char *cainfo, const char *capath,
 					continue;
 				}
 				i = submit_or_poll_uri(uris[u], cainfo, capath,
-						       csr, reqprinc, profile);
+						       uid, pwd, csr, reqprinc,
+						       profile, issuer);
 				if ((i != CM_SUBMIT_STATUS_UNREACHABLE) &&
 				    (i != CM_SUBMIT_STATUS_UNCONFIGURED)) {
 					talloc_free(uris);
@@ -471,7 +500,8 @@ submit_or_poll(const char *uri, const char *cainfo, const char *capath,
 
 static int
 fetch_roots(const char *server, int ldap_uri_cmd, const char *ldap_uri,
-	    const char *host, const char *domain, char *basedn)
+	    const char *host, const char *uid, const char *pwd,
+	    const char *domain, char *basedn)
 {
 	char *realm = NULL;
 	LDAP *ld = NULL;
@@ -553,10 +583,11 @@ main(int argc, const char **argv)
 	const char *host = NULL, *domain = NULL, *cainfo = NULL, *capath = NULL;
 	const char *ktname = NULL, *kpname = NULL;
 	char *csr, *p, uri[LINE_MAX], *reqprinc = NULL, *ipaconfig, *kerr;
+	char *uid = NULL, *pwd = NULL, *pwdfile = NULL;
 	const char *xmlrpc_uri = NULL, *ldap_uri = NULL, *server = NULL, *csrfile;
 	int xmlrpc_uri_cmd = 0, ldap_uri_cmd = 0, verbose = 0;
 	const char *mode = CM_OP_SUBMIT;
-	char ldn[LINE_MAX], *basedn = NULL, *profile = NULL;
+	char ldn[LINE_MAX], *basedn = NULL, *profile = NULL, *issuer = NULL;
 	krb5_error_code kret;
 	poptContext pctx;
 	struct poptOption popts[] = {
@@ -569,8 +600,12 @@ main(int argc, const char **argv)
 		{"keytab-name", 't', POPT_ARG_STRING, NULL, 't', "location of credentials to use for authenticating to server", "KEYTAB"},
 		{"submitter-principal", 'k', POPT_ARG_STRING, &kpname, 'k', "principal name to use for authenticating to server", "PRINCIPAL"},
 		{"use-ccache-creds", 'K', POPT_ARG_NONE, NULL, 'K', "use default ccache instead of creating a new one using keytab", NULL},
+		{"uid", 'u', POPT_ARG_STRING, &uid, 0, "authenticate to server using Basic authentication", "USERNAME"},
+		{"pwd", 'W', POPT_ARG_STRING, &pwd, 0, "password to use when using Basic authentication", "PASSWORD"},
+		{"pwdfile", 'w', POPT_ARG_STRING, &pwdfile, 0, "read password from file for Basic authentication", "FILENAME"},
 		{"principal-of-request", 'P', POPT_ARG_STRING, &reqprinc, 0, "principal name in signing request", "PRINCIPAL"},
 		{"profile", 'T', POPT_ARG_STRING, &profile, 0, "request enrollment using the specified profile", "NAME"},
+		{"issuer", 'X', POPT_ARG_STRING, &issuer, 0, "request enrollment using the specified CA", "NAME"},
 		{"basedn", 'b', POPT_ARG_STRING, &basedn, 0, "IPA domain LDAP base DN", "DN"},
 		{"verbose", 'v', POPT_ARG_NONE, NULL, 'v', NULL, NULL},
 		POPT_AUTOHELP
@@ -729,6 +764,10 @@ main(int argc, const char **argv)
 		    (getenv(CM_SUBMIT_PROFILE_ENV) != NULL)) {
 			profile = strdup(getenv(CM_SUBMIT_PROFILE_ENV));
 		}
+		if ((issuer == NULL) &&
+		    (getenv(CM_SUBMIT_ISSUER_ENV) != NULL)) {
+			issuer = strdup(getenv(CM_SUBMIT_ISSUER_ENV));
+		}
 		if ((server != NULL) && !xmlrpc_uri_cmd) {
 			snprintf(uri, sizeof(uri),
 				 "https://%s/ipa/xml", server);
@@ -753,7 +792,13 @@ main(int argc, const char **argv)
 			}
 		}
 		if ((csr == NULL) || (strlen(csr) == 0)) {
-			printf(_("Unable to read signing request.\n"));
+			if (csrfile != NULL) {
+				printf(_("Unable to read signing request from file \"%s\".\n"),
+				       csrfile);
+			} else {
+				printf(_("Unable to read signing request from environment variable \"%s\".\n"),
+				       CM_SUBMIT_CSR_ENV);
+			}
 			poptPrintUsage(pctx, stdout, 0);
 			return CM_SUBMIT_STATUS_UNCONFIGURED;
 		}
@@ -789,6 +834,29 @@ main(int argc, const char **argv)
 		}
 	}
 
+	/* If we're supposed to read a password from a file, read it now. */
+	if ((pwdfile != NULL) && (pwd == NULL)) {
+		pwd = cm_submit_u_from_file(pwdfile);
+		if (pwd == NULL) {
+			fprintf(stderr,
+				"Error reading password from \"%s\": %s.\n",
+				pwdfile, strerror(errno));
+			return CM_SUBMIT_STATUS_UNCONFIGURED;
+		}
+	}
+
+	/* If we're using Basic auth, make sure we don't try to set up a
+	 * ccache. */
+	if ((uid != NULL) && (pwd != NULL)) {
+		make_keytab_ccache = FALSE;
+	} else {
+		if ((uid != NULL) || (pwd != NULL)) {
+			fprintf(stderr,
+				"Both -u and -W/-w options should be specified.\n");
+			return CM_SUBMIT_STATUS_UNCONFIGURED;
+		}
+	}
+
 	/* Setup a ccache unless we're told to use the default one. */
 	kerr = NULL;
 	if (make_keytab_ccache &&
@@ -832,14 +900,14 @@ main(int argc, const char **argv)
 
 	if ((strcasecmp(mode, CM_OP_SUBMIT) == 0) ||
 	    (strcasecmp(mode, CM_OP_POLL) == 0)) {
-		return submit_or_poll(uri, cainfo, capath,
-				      server, ldap_uri_cmd, ldap_uri,
-				      host, domain, basedn,
-				      csr, reqprinc, profile);
+		return submit_or_poll(uri, cainfo, capath, server,
+				      ldap_uri_cmd, ldap_uri, host, domain,
+				      basedn, uid, pwd, csr, reqprinc, profile,
+				      issuer);
 	} else
 	if (strcasecmp(mode, CM_OP_FETCH_ROOTS) == 0) {
 		return fetch_roots(server, ldap_uri_cmd, ldap_uri, host,
-				   domain, basedn);
+				   uid, pwd, domain, basedn);
 	}
 
 	return CM_SUBMIT_STATUS_OPERATION_NOT_SUPPORTED;
