@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011,2012,2013,2014,2015 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012,2013,2014,2015,2017 Red Hat, Inc.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,10 +33,25 @@
 #include <dbus/dbus.h>
 #include <talloc.h>
 
+#include <krb5.h>
+
+#ifdef HAVE_UUID
+#if defined(HAVE_UUID_H)
+#include <uuid.h>
+#elif defined(HAVE_UUID_UUID_H)
+#include <uuid/uuid.h>
+#endif
+#endif
+
+#ifdef HAVE_GMP
+#include <gmp.h>
+#endif
+
 #include "env.h"
 #include "store.h"
 #include "store-int.h"
 #include "submit-e.h"
+#include "submit-u.h"
 #include "log.h"
 #include "tm.h"
 
@@ -129,6 +144,7 @@ enum cm_store_file_field {
 	cm_store_entry_field_template_ocsp_location,
 	cm_store_entry_field_template_ns_comment,
 	cm_store_entry_field_template_profile,
+	cm_store_entry_field_template_issuer,
 	cm_store_entry_field_template_no_ocsp_check,
 	cm_store_entry_field_template_ns_certtype,
 
@@ -303,6 +319,7 @@ static struct cm_store_file_field_list {
 	{cm_store_entry_field_template_ns_comment, "template_ns_comment"},
 	{cm_store_entry_field_template_profile, "template_profile"}, /* right */
 	{cm_store_entry_field_template_profile, "ca_profile"}, /* wrong */
+	{cm_store_entry_field_template_issuer, "template_issuer"},
 	{cm_store_entry_field_template_no_ocsp_check, "template_no_ocsp_check"},
 	{cm_store_entry_field_template_ns_certtype, "template_ns_certtype"},
 
@@ -669,6 +686,70 @@ parse_nickcert_list(void *parent, const char *s)
 	return ret;
 }
 
+#ifdef HAVE_GMP
+static void
+fill_uuid(unsigned char *uuid, size_t length)
+{
+	gmp_randstate_t state;
+	unsigned int i;
+
+	gmp_randinit_default(state);
+	for (i = 0; i < length; i++) {
+		uuid[i] = gmp_urandomb_ui(state, 8);
+	}
+}
+#endif
+
+int
+cm_store_make_uuid_string(char out[37])
+{
+	unsigned char uuid[16];
+	char *p;
+	int i;
+
+#ifdef HAVE_UUID
+	if (cm_submit_uuid_new(uuid) == 0) {
+		/* we're good */
+	} else
+#endif
+#ifdef HAVE_GMP
+	fill_uuid(uuid, sizeof(uuid));
+#else
+	return -1;
+#endif
+	for (i = 0, p = out; i < 16; i++) {
+		switch (i) {
+		case 4:
+		case 6:
+		case 8:
+		case 10:
+			*p++ = '-';
+			break;
+		default:
+			break;
+		}
+		snprintf(p, 3, "%02x", uuid[i]);
+		p += 2;
+	}
+	return p - out;
+}
+
+int
+cm_store_make_uuid_string_underscore(char out[37])
+{
+	int i, j;
+
+	i = cm_store_make_uuid_string(out);
+	if (i > 0) {
+		for (j = 0; j < i; j++) {
+			if (out[j] == '-') {
+				out[j] = '_';
+			}
+		}
+	}
+	return i;
+}
+
 char *
 cm_store_entry_next_busname(void *parent)
 {
@@ -1127,6 +1208,9 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_template_profile:
 				ret->cm_template_profile = free_if_empty(p);
 				break;
+			case cm_store_entry_field_template_issuer:
+				ret->cm_template_issuer = free_if_empty(p);
+				break;
 			case cm_store_entry_field_template_no_ocsp_check:
 				ret->cm_template_no_ocsp_check = atoi(p) != 0;
 				talloc_free(p);
@@ -1370,6 +1454,7 @@ cm_store_ca_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_template_ocsp_location:
 			case cm_store_entry_field_template_ns_comment:
 			case cm_store_entry_field_template_profile:
+			case cm_store_entry_field_template_issuer:
 			case cm_store_entry_field_template_no_ocsp_check:
 			case cm_store_entry_field_template_ns_certtype:
 			case cm_store_entry_field_challenge_password:
@@ -1972,6 +2057,8 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 				entry->cm_template_ns_comment);
 	cm_store_file_write_str(fp, cm_store_entry_field_template_profile,
 				entry->cm_template_profile);
+	cm_store_file_write_str(fp, cm_store_entry_field_template_issuer,
+				entry->cm_template_issuer);
 	cm_store_file_write_int(fp, cm_store_entry_field_template_no_ocsp_check,
 				entry->cm_template_no_ocsp_check ? 1 : 0);
 	cm_store_file_write_str(fp, cm_store_entry_field_template_ns_certtype,
@@ -2735,6 +2822,7 @@ cm_store_entry_dup(void *parent, struct cm_store_entry *entry)
 	ret->cm_template_ocsp_location = cm_store_maybe_strdupv(ret, entry->cm_template_ocsp_location);
 	ret->cm_template_ns_comment = cm_store_maybe_strdup(ret, entry->cm_template_ns_comment);
 	ret->cm_template_profile = cm_store_maybe_strdup(ret, entry->cm_template_profile);
+	ret->cm_template_issuer = cm_store_maybe_strdup(ret, entry->cm_template_issuer);
 	ret->cm_template_no_ocsp_check = entry->cm_template_no_ocsp_check;
 	ret->cm_template_ns_certtype = cm_store_maybe_strdup(ret,
 							     entry->cm_template_ns_certtype);
