@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Red Hat, Inc.
+ * Copyright (C) 2009,2011,2012,2014,2015 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +39,8 @@
 #define CM_DBUS_PROP_NICKNAME "nickname"
 #define CM_DBUS_PROP_AUTORENEW "autorenew"
 #define CM_DBUS_PROP_CERT "cert"
+#define CM_DBUS_PROP_CERT_CHAIN "cert-chain"
+#define CM_DBUS_PROP_CERT_ROOTS "cert-roots"
 #define CM_DBUS_PROP_CERT_PRESAVE_COMMAND "cert-presave-command"
 #define CM_DBUS_PROP_CERT_PRESAVE_UID "cert-presave-uid"
 #define CM_DBUS_PROP_CERT_POSTSAVE_COMMAND "cert-postsave-command"
@@ -46,6 +48,8 @@
 #define CM_DBUS_PROP_CERT_ISSUER "issuer"
 #define CM_DBUS_PROP_CERT_SERIAL "serial"
 #define CM_DBUS_PROP_CERT_SUBJECT "subject"
+#define CM_DBUS_PROP_CERT_NOT_VALID_BEFORE "not-valid-before"
+#define CM_DBUS_PROP_CERT_NOT_VALID_AFTER "not-valid-after"
 #define CM_DBUS_PROP_CERT_EMAIL "email"
 #define CM_DBUS_PROP_CERT_KU "ku"
 #define CM_DBUS_PROP_CERT_EKU "eku"
@@ -57,6 +61,8 @@
 #define CM_DBUS_PROP_CERT_LOCATION_DATABASE "cert-database"
 #define CM_DBUS_PROP_CERT_LOCATION_NICKNAME "cert-nickname"
 #define CM_DBUS_PROP_CERT_LOCATION_TOKEN "cert-token"
+#define CM_DBUS_PROP_CERT_OWNER "cert-owner"
+#define CM_DBUS_PROP_CERT_PERMS "cert-perms"
 #define CM_DBUS_PROP_CSR "csr"
 #define CM_DBUS_PROP_TEMPLATE_SUBJECT "template-subject"
 #define CM_DBUS_PROP_TEMPLATE_EMAIL "template-email"
@@ -64,11 +70,15 @@
 #define CM_DBUS_PROP_TEMPLATE_EKU "template-eku"
 #define CM_DBUS_PROP_TEMPLATE_HOSTNAME "template-hostname"
 #define CM_DBUS_PROP_TEMPLATE_PRINCIPAL "template-principal"
+#define CM_DBUS_PROP_TEMPLATE_CHALLENGE_PASSWORD "template-challenge-password"
+#define CM_DBUS_PROP_TEMPLATE_CHALLENGE_PASSWORD_FILE "template-challenge-password-file"
 #define CM_DBUS_PROP_KEY_LOCATION_TYPE "key-storage"
 #define CM_DBUS_PROP_KEY_LOCATION_FILE "key-file"
 #define CM_DBUS_PROP_KEY_LOCATION_DATABASE "key-database"
 #define CM_DBUS_PROP_KEY_LOCATION_NICKNAME "key-nickname"
 #define CM_DBUS_PROP_KEY_LOCATION_TOKEN "key-token"
+#define CM_DBUS_PROP_KEY_OWNER "key-owner"
+#define CM_DBUS_PROP_KEY_PERMS "key-perms"
 #define CM_DBUS_PROP_KEY_TYPE "key-type"
 #define CM_DBUS_PROP_KEY_SIZE "key-size"
 #define CM_DBUS_PROP_MONITORING "monitoring"
@@ -78,6 +88,8 @@
 #define CM_DBUS_PROP_NOTIFICATION_COMMAND "notification-command"
 #define CM_DBUS_PROP_KEY_PIN_FILE "key-pin-file"
 #define CM_DBUS_PROP_KEY_PIN "key-pin"
+#define CM_DBUS_PROP_KEY_GEN_DATE "key-generated-date"
+#define CM_DBUS_PROP_KEY_ISSUED_COUNT "key-issued-count"
 #define CM_DBUS_PROP_STATUS "status"
 #define CM_DBUS_PROP_STUCK "stuck"
 #define CM_DBUS_PROP_CA "ca"
@@ -86,17 +98,49 @@
 #define CM_DBUS_PROP_CA_ERROR "ca-error"
 #define CM_DBUS_PROP_SUBMITTED_DATE "submitted-date"
 #define CM_DBUS_PROP_IS_DEFAULT "is-default"
+#define CM_DBUS_PROP_EXTERNAL_HELPER "external-helper"
 #define CM_DBUS_PROP_ISSUER_NAMES "issuer-names"
+#define CM_DBUS_PROP_TEMPLATE_IP_ADDRESS "template-ip-address"
 #define CM_DBUS_PROP_TEMPLATE_IS_CA "template-is-ca"
 #define CM_DBUS_PROP_TEMPLATE_CA_PATH_LENGTH "template-ca-path-length"
 #define CM_DBUS_PROP_TEMPLATE_OCSP "template-ocsp"
 #define CM_DBUS_PROP_TEMPLATE_CRL_DP "template-crldp"
+#define CM_DBUS_PROP_TEMPLATE_FRESHEST_CRL "template-freshest-crl"
 #define CM_DBUS_PROP_TEMPLATE_NS_COMMENT "template-ns-comment"
 #define CM_DBUS_PROP_TEMPLATE_PROFILE "template-profile"
+#define CM_DBUS_PROP_TEMPLATE_ISSUER "template-issuer"
+#define CM_DBUS_PROP_TEMPLATE_NS_CERTTYPE "template-ns-certtype"
 #define CM_DBUS_SIGNAL_REQUEST_CERT_SAVED "SavedCertificate"
+#define CM_DBUS_PROP_CA_PRESAVE_COMMAND "ca-presave-command"
+#define CM_DBUS_PROP_CA_PRESAVE_UID "ca-presave-uid"
+#define CM_DBUS_PROP_CA_POSTSAVE_COMMAND "ca-postsave-command"
+#define CM_DBUS_PROP_CA_POSTSAVE_UID "ca-postsave-uid"
+#define CM_DBUS_PROP_ROOT_CERTS "root-certs"
+#define CM_DBUS_PROP_OTHER_ROOT_CERTS "root-other-certs"
+#define CM_DBUS_PROP_OTHER_CERTS "other-certs"
+#define CM_DBUS_PROP_SCEP_CA_IDENTIFIER "scep-ca-identifier"
+#define CM_DBUS_PROP_SCEP_CA_CAPABILITIES "scep-ca-capabilities"
+#define CM_DBUS_PROP_SCEP_RA_CERT "scep-ra-cert"
+#define CM_DBUS_PROP_SCEP_CA_CERT "scep-ca-cert"
+#define CM_DBUS_PROP_SCEP_OTHER_CERTS "scep-other-certs"
+#define CM_DBUS_PROP_ROOT_CERT_FILES "root-cert-files"
+#define CM_DBUS_PROP_OTHER_ROOT_CERT_FILES "root-other-cert-files"
+#define CM_DBUS_PROP_OTHER_CERT_FILES "other-cert-files"
+#define CM_DBUS_PROP_ROOT_CERT_NSSDBS "root-cert-nssdbs"
+#define CM_DBUS_PROP_OTHER_ROOT_CERT_NSSDBS "root-other-cert-nssdbs"
+#define CM_DBUS_PROP_OTHER_CERT_NSSDBS "other-cert-nssdbs"
+#define CM_DBUS_PROP_SUPPORTED_PROFILES "supported-profiles"
+#define CM_DBUS_PROP_DEFAULT_PROFILE "default-profile"
+#define CM_DBUS_PROP_REQUIRED_ENROLL_ATTRIBUTES "required-enroll-attributes"
+#define CM_DBUS_PROP_REQUIRED_RENEW_ATTRIBUTES "required-renew-attributes"
+#define CM_DBUS_PROP_AKA "aka"
 
-enum cm_tdbus_type { cm_tdbus_system, cm_tdbus_session };
-int cm_tdbus_setup(struct tevent_context *ec, enum cm_tdbus_type bus_type,
-		   void *data, DBusError *error);
+enum cm_tdbus_type { cm_tdbus_system, cm_tdbus_session, cm_tdbus_private };
+int cm_tdbus_setup_public(struct tevent_context *ec,
+			  enum cm_tdbus_type bus_type,
+			  void *data, DBusError *error);
+int cm_tdbus_setup_private(struct tevent_context *ec, void *data,
+			   const char *socket_path,
+			   char **address, DBusError *error);
 
 #endif
